#!/usr/bin/env python
# -*- coding: utf-8 -*-

__licence__ = """Copyright (C) 2009 Loïc Pefferkorn
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import subprocess
import os
import os.path
import signal
import re
import threading
import time
import sys
import gettext
import locale
import getopt
import urllib2

try:
	import wxversion
	wxversion.select("2.8")
	import wx
	_ = wx.GetTranslation	# Used for strings translations
except:
	print _('wxPython-2.8 module is required to run this program.')
	sys.exit(1)

__version__ = '1.1'
__date__ = '26/06/2009'
__icon_file__ = 'realripper.png'


__mplayer_binary_paths__ = ['/usr/bin/mplayer', '/usr/local/bin/mplayer']
__mplayer_thread_check_interval__ = 1000

def findMplayerBinary():
	""" Try to autodetect mplayer binary among a set of predefined paths
	"""
	for e in __mplayer_binary_paths__:
		if os.path.exists(e):
			return e
	return ''

def errorDialog(msg):
	""" Display an error dialog
	"""
	errorDlg = wx.MessageDialog(None, 
			message=msg, caption=_('Error'),style=wx.OK|wx.ICON_ERROR)
	if errorDlg.ShowModal() == wx.ID_OK:
		errorDlg.Destroy()

	
def isStreamAddressValid(line):
	""" Return True if the given line looks like a valid rtsp:// address
	"""
	if re.match('^rtsp://', line) is None:
		return False
	else:
		return True

class MyFrame(wx.Frame):
	
	def handleCommandLineArguments(self, fileArg, urlArg, streamArg):
		""" Process command line arguments
		"""
		if (streamArg is not None):
			if (isStreamAddressValid(streamArg) is True):
				self.streamTxt.SetValue(streamArg)
			else:
				errorDialog(_('Stream address is not a valid rtsp one !\n\n%s') % streamArg)
				self.streamTxt.SetValue('')
				self.streamTxt.SetFocus()
		# Local .ram file
		elif (fileArg is not None):
			if (os.path.exists(fileArg) is True):
				try:
					line = open(fileArg, 'r').readline().rstrip('\r\n')
				except IOError, e:
					errorDialog(_('Error while reading .ram file !\n\n%s') % fileArg)
					self.streamTxt.SetValue('')
					self.streamTxt.SetFocus()
					
				if (isStreamAddressValid(line) is True):
					self.streamTxt.SetValue(line)
				else:
					errorDialog(_('This .ram file does not contain a valid rtsp address !\n\n%s') % fileArg)
					self.streamTxt.SetValue('')
					self.streamTxt.SetFocus()
			else:
				errorDialog(_('This .ram file does not exist !\n\n%s') % fileArg)
				self.streamTxt.SetValue('')
				self.streamTxt.SetFocus()
		# remote http .ram file
		elif (urlArg is not None):
			req = urllib2.Request(urlArg)
			req.add_header('User-agent', 'Realripper %s' % __version__)
			try:
				r = urllib2.urlopen(req)
				line = r.readline().rstrip('\r\n')
			except urllib2.URLError, e:
				errorDialog(_('Cannot open remote .ram file !\n%s\n\n%s') % (e,urlArg))
				self.streamTxt.SetValue('')
				self.streamTxt.SetFocus()
				return
				
			if (isStreamAddressValid(line) is True):
				self.streamTxt.SetValue(line)
			else:
				errorDialog(_('Remote .ram file does not contain a valid rtsp address !\n\n%s') % urlArg)
				self.streamTxt.SetValue('')
				self.streamTxt.SetFocus()
	
	
	def __init__(self, parent=None, id=-1, title='RealRipper '+ __version__, fileArg=None, urlArg=None, streamArg=None):
		wx.Frame.__init__(self, parent, id, title, pos=wx.DefaultPosition)
		
		#print ('MyFrame() file:%s url:%s stream:%s' % (fileArg, urlArg, streamArg))
		self.SetIcon(wx.Icon(__icon_file__, wx.BITMAP_TYPE_PNG))
		self.Bind(wx.EVT_CLOSE, self.OnQuit)

		# Menu setup
		menuFile = wx.Menu()
		menuFile.Append(1, _('&Quit'))
		menuFile.Append(2, _('&About...'))
		menuBar = wx.MenuBar()
		menuBar.Append(menuFile, _('&File'))
		self.SetMenuBar(menuBar)
		self.Bind(wx.EVT_MENU, self.OnQuit, id=1)
		self.Bind(wx.EVT_MENU, self.OnAbout, id=2)

		self.statusBar = self.CreateStatusBar()
		self.statusBar.SetFieldsCount(2)
		self.statusBar.SetStatusText(_('Idle'))

		# Settings section
		self.mplayerLbl = wx.StaticText(self, -1, _('MPlayer binary path (autodetected)'))
		self.mplayerTxt = wx.TextCtrl(self, -1, size=(200, -1))
		self.mplayerBtn = wx.Button(self, -1, _('Browse...'))

		self.mplayerBtn.Bind(wx.EVT_BUTTON, self.OnMPlayerBtnClicked)
		self.mplayerTxt.Bind(wx.EVT_TEXT, self.OnCtrlTxtChange)
		self.mplayerTxt.SetToolTip(wx.ToolTip(_('MPlayer binary, usually "mplayer" file in bin directories')))


		sizer1 = wx.GridBagSizer(hgap=5, vgap=5)
		sizer1.Add(self.mplayerLbl, pos=(0,0))
		sizer1.Add(self.mplayerTxt, pos=(1,0), flag=wx.EXPAND)
		sizer1.Add(self.mplayerBtn, pos=(1,1))
		sizer1.AddGrowableCol(0,0)

		settingsBox = wx.StaticBox(self, -1, _('Settings'))
		settingsSizer = wx.StaticBoxSizer(settingsBox, wx.VERTICAL)
		settingsSizer.Add(sizer1, proportion=0, flag=wx.ALL|wx.EXPAND, border=10)

		# Stream section
		self.streamLbl = wx.StaticText(self, -1, _('Stream url'))
		self.streamTxt = wx.TextCtrl(self, -1, size=(200, -1))
		streamBtn = wx.Button(self, -1, _('Open a .ram file...'))

		streamBtn.Bind(wx.EVT_BUTTON, self.OnStreamBtnClicked)
		self.streamTxt.Bind(wx.EVT_TEXT, self.OnCtrlTxtChange)
		self.streamTxt.SetToolTip(wx.ToolTip(_('Address beginning with rtsp://')))

		streamSizer = wx.GridBagSizer(hgap=5, vgap=5)
		streamSizer.Add(self.streamLbl, pos=(0,0))
		streamSizer.Add(self.streamTxt, pos=(1,0), flag=wx.EXPAND)
		streamSizer.Add(streamBtn, pos=(1,1))
		streamSizer.AddGrowableCol(0,0)

		self.dumpLbl = wx.StaticText(self, -1, _('Output file'))
		self.dumpTxt = wx.TextCtrl(self, -1, size=(200, -1))
		self.dumpBtn = wx.Button(self, -1, _('Browse...'))

		self.dumpBtn.Bind(wx.EVT_BUTTON, self.OnDumpBtnClicked)
		self.dumpTxt.Bind(wx.EVT_TEXT, self.OnCtrlTxtChange)
		self.dumpTxt.SetToolTip(wx.ToolTip(_('File where the stream should be recorded to')))

		dumpSizer = wx.GridBagSizer(hgap=5, vgap=5)
		dumpSizer.Add(self.dumpLbl, pos=(0,0))
		dumpSizer.Add(self.dumpTxt, pos=(1,0), flag=wx.EXPAND)
		dumpSizer.Add(self.dumpBtn, pos=(1,1))
		dumpSizer.AddGrowableCol(0,0)

		fluxBox = wx.StaticBox(self, -1, _('Stream'))
		fluxSizer = wx.StaticBoxSizer(fluxBox, wx.VERTICAL)
		fluxSizer.Add(streamSizer, proportion=0, flag=wx.ALL|wx.EXPAND, border=10)
		fluxSizer.Add(dumpSizer, proportion=0, flag=wx.ALL|wx.EXPAND, border=10)

		# Recording section
		self.startBtn = wx.Button(self, -1, _('Start'))
		self.startBtn.Enable(False)
		self.stopBtn = wx.Button(self, -1, _('Stop'))
		self.stopBtn.Enable(False)

		self.startBtn.Bind(wx.EVT_BUTTON, self.OnStartBtnClicked)
		self.stopBtn.Bind(wx.EVT_BUTTON, self.OnStopBtnClicked)

		enregistrementBox = wx.StaticBox(self, -1, _('Recording'))
		enregistrementSizer = wx.StaticBoxSizer(enregistrementBox, wx.HORIZONTAL)
		enregistrementSizer.Add(self.startBtn, proportion=0, flag=wx.ALL, border=10)
		enregistrementSizer.Add(self.stopBtn, proportion=0, flag=wx.ALL, border=10)


		mainSizer = wx.BoxSizer(wx.VERTICAL)

		sizer = wx.BoxSizer(wx.VERTICAL)
		sizer.Add(settingsSizer, proportion=0, flag=wx.EXPAND|wx.TOP, border=5)
		sizer.Add(fluxSizer, proportion=0, flag=wx.EXPAND|wx.TOP, border=5)
		sizer.Add(enregistrementSizer, proportion=0, flag=wx.EXPAND|wx.TOP, border=5)

		mainSizer.Add(sizer, 1, wx.ALL|wx.EXPAND, 15)
		self.SetSizer(mainSizer)
		mainSizer.Fit(self)
		mainSizer.SetSizeHints(self)

		# GUI is now fully built
		
		# Fill GUI fields with command line parameters
		self.handleCommandLineArguments(fileArg, urlArg, streamArg)

		# Automatic detection of MPlayer binary
		self.mplayerPath = findMplayerBinary()
		self.mplayerTxt.SetValue(self.mplayerPath)

		# Used to check status of the thread executing MPlayer
		self.timer = wx.Timer(self)
		self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer)

		# MPlayer exit code in thread
		self.MPlayerReturnCode = None

	
	def OnMPlayerBtnClicked(self, event):
		dlg = wx.FileDialog(self, message=_('Select Mplayer binary'), style=wx.OPEN|wx.FILE_MUST_EXIST)
		if dlg.ShowModal() == wx.ID_OK:
			self.MPlayer = dlg.GetPath()
			self.mplayerTxt.SetValue(self.MPlayer)
			dlg.Destroy()

	def OnStreamBtnClicked(self, event):
		dlg = wx.FileDialog(self, message=_('Select .ram file'), style=wx.OPEN|wx.FILE_MUST_EXIST,
			wildcard=_('Real files (*.ram;*.rm)|*.ram;*.rm)'))
		if dlg.ShowModal() == wx.ID_OK:
			try:
				line = open(dlg.GetPath(), 'r').readline().rstrip('\r\n')
			except IOError, e:
				errorDialog(_('Error while reading .ram file !\n\n%s') % fileArg)
				self.streamTxt.SetValue('')
				self.streamTxt.SetFocus()
			
			if (isStreamAddressValid(line) is not False):
				self.streamTxt.SetValue(line)
			else:
				errorDialog(_('This .ram file does not contain a valid rtsp address !\n\n%s') % dlg.GetPath())
				self.streamTxt.SetValue('')
				self.streamTxt.SetFocus()
		dlg.Destroy()
		
	def OnDumpBtnClicked(self, event):
		dlg = wx.FileDialog(self, message=_('Select output file'), style=wx.SAVE|wx.OVERWRITE_PROMPT,
			wildcard=_('Real files(*.rm)|*.rm'))
		if dlg.ShowModal() == wx.ID_OK:
			self.dumpTxt.SetValue(dlg.GetPath().rstrip('\r\n'))
		dlg.Destroy()

	def OnCtrlTxtChange(self, event):
		if self.mplayerTxt.GetValue() != '' and self.streamTxt.GetValue() != ''	and self.dumpTxt.GetValue() != '':
			self.startBtn.Enable(True)
		else:
			self.startBtn.Enable(False) 

	def OnTimer(self, event):
		""" Check if the thread executing MPlayer is still running, 
			and update current ouput file's size
		"""
		if self.mpThread.isAlive():
			filesize = os.stat(self.dumpTxt.GetValue()).st_size
			self.statusBar.SetStatusText(_('Recording...'), 0)
			self.statusBar.SetStatusText(_('File size: %s') % str(filesize/1024) +_(' Kb'), 1)
		else:
			#print 'exit code:' + str(self.MPlayerReturnCode)
			if self.MPlayerReturnCode == 0:
				self.statusBar.SetStatusText(_('Successfuly recorded'))
			else:
				self.statusBar.SetStatusText(_('Error ! MPlayer exit code %s') % str(self.MPlayerReturnCode))
			self.timer.Stop()
			self.startBtn.Enable(True)
			self.stopBtn.Enable(False)
			self.dumpTxt.Enable(True)
			self.streamTxt.Enable(True)

	def OnStartBtnClicked(self, event):
		
		if (isStreamAddressValid( self.streamTxt.GetValue()) is None):
			errorDialog(_('Stream address is not a valid rtsp one !\n\n%s') % self.streamTxt.GetValue())
			self.streamTxt.SetFocus()
			return
			
		#Warn the user if the output file already exists
		if (os.path.exists(self.dumpTxt.GetValue())):
			dlg = wx.MessageDialog(self, message = _('Output file already exists, overwrite ?'),
			caption=_('Information'), style=wx.YES_NO|wx.ICON_QUESTION)
			if (dlg.ShowModal() == wx.ID_NO):
				return

		# Try to open output file for writing
		try:
			i = open(self.dumpTxt.GetValue(), 'w')
			i.close()
		except IOError, e:
			errorDialog(_('Unable to open output file for writing !\n\n%s') % self.dumpTxt.GetVAlue())
			self.dumpTxt.SetFocus()
			return

		self.mpThreadReturnCode = None
		self.mpThread = MPlayerThread(self, self.mplayerTxt.GetValue(), self.streamTxt.GetValue(), self.dumpTxt.GetValue())
		self.mpThread.start()
		self.timer.Start(__mplayer_thread_check_interval__)
		self.startBtn.Enable(False)
		self.stopBtn.Enable(True)
		self.streamTxt.Enable(False)
		self.dumpTxt.Enable(False)


	def OnStopBtnClicked(self, event):
		self.mpThread.stop()  

	def OnQuit(self, event):
		# Ensure MPlayer process is killed on exit 
		try:
			if self.mpThread.isAlive() is True:
				#print 'MPlayer was running, killing it...'
				self.mpThread.stop()
		except:
			pass
		self.Destroy()

	def OnAbout(self, event):
		info = wx.AboutDialogInfo()
		info.SetIcon(wx.Icon(__icon_file__, wx.BITMAP_TYPE_PNG))
		info.SetName('RealRipper')
		info.SetVersion(__version__)
		info.SetDescription(_('RealRipper records Real streams to files.\nThis program uses MPlayer'))
		info.SetCopyright('(c) 2009 Loïc Pefferkorn')
		info.SetWebSite('http://www.loicp.eu/realripper')
		info.SetLicence(__licence__)
		info.AddDeveloper('Loïc Pefferkorn <loic-realripper@loicp.eu>')
		info.AddTranslator('fr - Loïc Pefferkorn <loic-realripper@loicp.eu>')
		
		wx.AboutBox(info)
	
	def LogMessage(self, mes):
		print 'Thread mes:' + str(mes)


class MPlayerThread(threading.Thread):
	def __init__(self, frame, MPlayerPath, url, filename):
		threading.Thread.__init__(self)
		self.frame = frame
		self.MPlayerPath = MPlayerPath
		self.url = url
		self.filename = filename
		self.running = True
		
	def stop(self):
		self.running = False

	def run(self):
		#wx.CallAfter(self.frame.LogMessage, 'debut')
		self.process = subprocess.Popen([self.MPlayerPath, '-dumpstream', self.url, '-dumpfile', self.filename])
		#self.process = subprocess.Popen(['ping', 'free.fr', '-c', '30'])
		#wx.CallAfter(self.frame.LogMessage, self.process.pid)

		while True:
			p = self.process.poll()
			self.frame.MPlayerReturnCode = p
			if p != None:
				#print 'mplayer exit code: '+str(p)
				#wx.CallAfter(self.frame.LogMessage, 'mplayer terminated:' +str(p))
				break
			
			if self.running is False:
				#wx.CallAfter(self.frame.LogMessage,  'mplayer kill requested')
				os.kill(self.process.pid, signal.SIGKILL) 
				self.process.wait()
				break
			time.sleep(1)

def version():
	print _('RealRipper version %s') % __version__

def usage():
	print _(
"""Realripper records Real stream to file.
Usage: realripper [OPTION]...

  -f ,  --file=FILE			path to local .ram file
  -u ,  --url=URL			http url to .ram file
  -s ,	--stream=STREAM		stream address (rtsp://)
  -v ,  --version			display version and exit
  -h,   --help				display this help and exit

Please report bugs to http://www.loicp.eu/realripper""")

class MyApp(wx.App):
	def OnInit(self):

		# command line arguments
		clFile = None
		clUrl = None
		clStream = None
		try:
			opts, args = getopt.getopt(sys.argv[1:], 'f:u:s:hv', 
			['file=', 'url=', 'stream=', 'help', 'version'])
    		except getopt.GetoptError:          
        		usage()                         
        		sys.exit(2)    
		for opt, arg in opts:
			if opt in ('-h', '--help'):
				usage()
				sys.exit() 
			elif opt in ('-v', '--version'):
				version()
				sys.exit()
			elif opt in ('-f', '--file'):
				clFile = arg
			elif opt in ('-u', '--url'):
				clUrl = arg
			elif opt in ('-s', '--stream'):
				clStream = arg

		self.frame = MyFrame(fileArg=clFile, urlArg=clUrl, streamArg=clStream)
		self.frame.Show()
		self.SetTopWindow(self.frame)
		return True


if __name__ == '__main__':
	curdir = os.path.dirname(os.path.abspath(__file__))
	localesdir = os.path.join(curdir, 'locale')
	locale = wx.Locale(wx.LANGUAGE_DEFAULT)
	#self.locale = wx.Locale(wx.LANGUAGE_FRENCH)
	#self.locale = wx.Locale(wx.LANGUAGE_GERMAN)
	#self.locale = wx.Locale(wx.LANGUAGE_ENGLISH_US)
	locale.AddCatalogLookupPathPrefix(localesdir)
	locale.AddCatalog('realripper')
	wx.SetDefaultPyEncoding("utf-8")

	app = MyApp()
	app.MainLoop()
